import java.awt.Cursor;
import javax.swing.Timer;
import java.awt.event.ActionListener;
import java.awt.event.ActionEvent;
import java.awt.Graphics2D;
import java.util.*;

public class MyWorld implements ActionListener {
   public static int WIDTH = 600;  // in pixels
   public static int HEIGHT = 500; // in pixels
   private double t;              // Simulation time
   private Timer pase;
   private double delta_t;        // in seconds
   private double refreshPeriod;  // in seconds
   private Ball selectedBall;
   private Vector2D g;
   private float viscosity;
   private PhysicsLabPanel labPanel;
   private PhysicsLabMenuBar menuBar;
   private GraphsPanel graphPanel;
   private Vector<PhysicsElement> elements;


   public MyWorld(){
      t = 0;
      refreshPeriod = 0.06;      // 80 [ms]
      delta_t = 0.0001;          // 0.1 [ms]
      pase = new Timer((int)(refreshPeriod*1000), this);
      g = new Vector2D(0, 0);
      viscosity = 0.0f;
      elements = new Vector<PhysicsElement>();
      selectedBall = null;
      labPanel = null;
      menuBar = null;
   }
   public void setPanel (PhysicsLabPanel p) {
      labPanel = p;
   }
   public void setMenuBar(PhysicsLabMenuBar mb){
       menuBar = mb;
   }
   public void setGraphPanel(GraphsPanel gp){
       graphPanel = gp;
   }
   public PhysicsLabMenuBar getMenuBar(){
       return menuBar;
   }
   public void setDelta_t(double delta) {
      delta_t = delta;
   }
   public void setRefreshPeriod (double rp) {
      refreshPeriod = rp;
      pase.setDelay((int)(refreshPeriod*1000));
   }
   public void setGravity (Vector2D gravity) {
      g = gravity;
   }
   public void setViscosity(float v){
      viscosity = v;
   }
   public void setCrossCursor(){
       labPanel.setCursor(Cursor.getPredefinedCursor(Cursor.CROSSHAIR_CURSOR));
       labPanel.deleteListener(true);
   }
   public void setNormalCursor(){
       labPanel.setCursor(Cursor.getPredefinedCursor(Cursor.DEFAULT_CURSOR));
       labPanel.deleteListener(false);
   }
   public void addElement(PhysicsElement e) {
       if(!pase.isRunning()){
        elements.add(e);
        labPanel.repaint();
       }
   }
   public void setBlock(int id){
   for (int i=0;i<elements.size();i++){
       if (elements.elementAt(i) instanceof Ball){
           if(elements.elementAt(i).getId()==id){
                if (selectedBall != elements.elementAt(i)){
                selectedBall = (Ball)elements.elementAt(i);
                graphPanel.newSelected(t);
                graphPanel.reset(t);
                }
            }
       }
   }
   }
   /**
 * Removes a physics element from the world.
 * @param e   Any sort of physics element (springs, rubber, ball, fixed hook)
 */
   public void removeElement(PhysicsElement e) {
       if(!pase.isRunning()){
      elements.remove(e);
      if (e instanceof Ball)
          menuBar.DeleteMassOption(e.getId());
      labPanel.repaint();
       }
   }
   /**
 * It starts the simulation. It restarts the simulation if a simulation has been
    * already started.
 */
   public void start() {
      if(pase.isRunning()) return;
      for (PhysicsElement e: elements){
         e.setInitialState(g, viscosity);
         if (e instanceof GBall){
             GBall ball = (GBall) e;
             ball.setSpeedZero();
         }
      }    
      setNormalCursor();
      labPanel.disableMouseListener();
      pase.start();
      graphPanel.reset(t);
   }
     /**
 * It resumes the simulation. It takes into account the previous state of the
      * physics element (unlike start()).
 */
   public void resume(){
       if(pase.isRunning()) return;
      for (PhysicsElement e: elements){
         e.setInitialState(g, viscosity);
         if (e instanceof Spring){
             Spring spring = (Spring) e;
             if (spring.getAelement()==null){
                 if (spring.getBelement() instanceof GBall){
                     GBall ball = (GBall)spring.getBelement();
                     ball.setSpeedZero();
                 }
             }
             if (spring.getBelement()==null){
                 if (spring.getAelement() instanceof GBall){
                     GBall ball = (GBall) spring.getAelement();
                     ball.setSpeedZero();
                 }
             }
         }
       }
      setNormalCursor();
      labPanel.disableMouseListener();
      pase.start();
   }
   public void stop(){
      pase.stop();
      labPanel.enableMouseListener();
   }
   public void actionPerformed (ActionEvent event) {  // simulate passing time
      double nextStop=t+refreshPeriod;
      for (; t<nextStop; t+=delta_t){
      for (PhysicsElement e: elements)
         e.computeNextState(delta_t, g, viscosity);
      for (PhysicsElement e: elements)
         e.updateState();
      }
      if (selectedBall != null)
          graphPanel.addSeries(selectedBall.getPosition(), t);
      labPanel.repaint();

   }
   public void paintComponent(Graphics2D g2d){
      for (PhysicsElement e: elements)
         if (e instanceof Drawable)
            ((Drawable)e).draw(g2d);
   }
   public Drawable find(int x, int y) {
      for (PhysicsElement e: elements)
         if (e instanceof Drawable){
            Drawable de = (Drawable) e;
            if (de.contains(x,y)) return de;
         }
      return null;
   }  

   public AttachableElement findAttachableElement(int x, int y) {
      for (PhysicsElement e :elements)
         if (e instanceof AttachableElement) {
            AttachableElement ae =(AttachableElement) e;
            if (ae instanceof Drawable)
            if (((Drawable)ae).contains(x,y)) return ae;
         }
      return null;
   }
} 